#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2013-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import sys
import matplotlib.pyplot as plt
import numpy as np
from cmath import exp
from zita_jacktools.jacksignal import JackSignal
sys.path.append ('..')
from utils.sinewave import *


# Harmonic distortion + noise measurement, 'analog style'.

# Run jack_nonlin 0.01 0.01 0.01
# Run jaaa and jnoisemeter

# This program
#
# 1. Creates a 1 kHz test signal, sends it to jack_nonlin
#    to add some distortion, and captures the result.
# 2. Removes the fundamental from the captured signal.
# 3. Plays the result to jaaa and jnoisemeter for
#    measurement and analysis. In jnoisemeter, use at
#    least the DC filter (even harmonics include DC).

# Test level (dB) 
#
Level = 0.0

# Test frequency (Hz) 
#
Freq = 1000


# Create a Jacksignal object.
#
J = JackSignal("THDtest")
if J.get_state() < 0:
    print ("Failed to create JackSignal -- is the server running ?")
    exit(1)

# Get Jack info.
#
name, Fsamp, period = J.get_jack_info()

# Create ports and connect.
#
J.create_output (0, "out_1")
J.create_output (1, "out_2")
J.create_input (0, "in_1")
J.silence()
J.connect_output (1, "jaaa:in_1")
J.connect_output (1, "jnoisemeter:in_1")
J.connect_output (0, "jack_nonlin:in")
J.connect_input (0, "jack_nonlin:out")

# Length of test signals.
#
siglen = int (Fsamp + 0.5) # One second
margin = siglen
buflen = siglen + margin

# Generate test signal, will be scaled for output.
#
Freq = adjust_freq (Freq, Fsamp, siglen)
Aout = gen_sinewave (1.00, Freq, Fsamp, buflen).astype (np.float32)

# Create input buffer.
#
Ainp = np.zeros ((siglen,), dtype = np.float32)

# Generate reference signal for selective measurement.
#
M1 = gen_complex (Freq, Fsamp, siglen, True)

# Define signal buffers and run test. 
# We skip the first half second.
#
amp = pow (10.0, Level / 20.0) 
J.set_output_data (0, amp * Aout)
J.set_input_data (0, Ainp, nskip = margin // 2) 
J.process()
J.wait()

# Subtract detected fundamental.    
#
M1 = gen_complex (Freq, Fsamp, siglen, False)
r, p = sigdetect (Ainp, M1)
a = r * exp ((0 -1j) * p)
Ainp -= (a * M1).real

# Output residual.
#
J.set_output_data (0, None)
J.set_output_data (1, Ainp, nloop = 1000)
J.set_input_data (0, None)
J.process()
J.wait()



# ----------------------------------------------------------------------------
#
#  Copyright (C) 2015-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from array import array
from zita_jacktools.jackclient import JackClient
from zita_jacktools import jacknmeter_ext


class JackNmeter(JackClient) :
    """
    Multichannel audio level meter with standard weighting
    filters and level detector ballistics. Each channel can
    have its own filter and/or detector type, and also provides
    the filtered input signal. Channels can also share an input.
    """

    FIL_NONE   = 0
    FIL_ENB20K = 1 # 20 kHz noise bandwidth
    FIL_IEC_A  = 2
    FIL_IEC_C  = 3
    FIL_ITU468 = 4
    FIL_DOLBY  = 5 # ITU468 with reduced gain
    
    DET_NONE      = 0
    DET_RMS       = 1
    DET_RMS_SLOW  = 2
    DET_VUM       = 3
    DET_VUM_SLOW  = 4
    DET_ITU468    = 5

    
    def __init__(self, ninp, nout, client_name, server_name = None):
        """
        Create a new JackNmeter instance. Initially no filter or
        detector is selected.

        The result should be checked using get_state().
        The optional 'server_name' allows to select between running
        Jack servers.
        """
        self._levels = array ('f', (0 for i in range (nout)))
        self._jnmeter, base = jacknmeter_ext.makecaps (self, client_name, server_name, ninp, nout, self._levels)
        super().__init__(base)
        
        
    def set_input(self, inp, out) :
        """
        Select input for given output, or for all outputs if 'out' < 0.
        """
        return jacknmeter_ext.set_input (self._jnmeter, inp, out)


    def set_filter(self, out, ftype, dcfilt = False) :
        """
        Set weighting and DC filters for output, or for all if 'out' < 0.
        """
        return jacknmeter_ext.set_filter (self._jnmeter, out, ftype, dcfilt)


    def set_detect(self, out, dtype) :
        """
        Set detector type for given output, or for all if 'out' < 0.
        """
        return jacknmeter_ext.set_detect (self._jnmeter, out, dtype)


    def get_levels(self) :
        """
        Return state and updated array of levels.

        Levels are on a linear scale, not in dB.
        """
        return jacknmeter_ext.get_levels (self._jnmeter), self._levels





#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2013-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import numpy as np
import matplotlib.pyplot as plt
from math import *
from time import sleep
from zita_jacktools.jackparameq import JackParameq
from zita_jacktools.jacksignal import JackSignal


# Measure high frequency shelf filter in JackParameq.
#

# Create and connect objects.
#
F = JackParameq (1, "H", "EQ")
if F.get_state() < 0:
    print ("Failed to create Jack client -- is Jack running ?")
    exit(1)

J = JackSignal("JackSignal")
J.create_output (0, "out-1")
J.create_input (0, "in-1")
J.silence()
J.connect_output (0, "EQ:in_0")
J.connect_input (0, "EQ:out_0")

# Get Jack info.
#
name, fsamp, period = J.get_jack_info ()

# Parameters.
#
fftlen = 16 * 1024

# Generate data.
#
Aout = np.zeros ([100,], dtype = np.float32)
Aout [0] = 1.0

Ain1 = np.empty ([fftlen,], dtype = np.float32)
Freq = np.linspace (0, fsamp / 2, num = fftlen // 2 + 1)

J.set_output_data (0, Aout)
J.set_input_data (0, Ain1, nskip = period)

# Set up plot.
#
fig = plt.figure (figsize=(10,6), facecolor='white')
fig.canvas.manager.set_window_title ('hfshelf')
xx = [0.04, 0.54, 0.04, 0.54]
yy = [0.54, 0.54, 0.04, 0.04]

freq = 5e3
F.set_bypass (False)

# Loop over gain range, shape = 0.
for i in range (3):
    ax = fig.add_axes ([xx[i], yy[i], 0.45, 0.45])
    ax.set_xlim (20, 20e3)
    ax.set_ylim (-25, 25)
    ax.set_xscale ('log')
    ax.grid ()
    s = 0.5 * i
    print ("s = %3.1f" % (s,))
    for g in range (-20,21,5):
        F.set_filter (0, freq,  g, s)
        # Wait for smooth change to end.
        if g == -20: sleep (0.5)
        else:        sleep (0.2)
        print ("   g = %5.1f" % (g,))
        # Measure.
        J.process()
        J.wait()
        # Plot result.
        Spec = np.fft.rfft (Ain1)
        if   g < 0: col = 'b'
        elif g > 0: col = 'r'
        else:       col = 'k'
        ax.plot (Freq, 20 * np.log10 (np.abs (Spec) + 1e-10), color = col, lw = 1)
    ax.text (500, 21, "freq = 5 kHz, shape = %3.1f" % (s,))

# Loop over shape range.
ax = fig.add_axes ([xx[3], yy[3], 0.45, 0.45])
ax.set_xlim (20, 20e3)
ax.set_ylim (-10, 25)
ax.set_xscale ('log')
ax.grid ()
g = 20.0
print ("g = %5.1f" % (g,))
for i in range (0, 11, 2):
    s = 0.1 * i
    F.set_filter (0, freq,  g, s)
    # Wait for smooth change to end.
    if i == 0: sleep (0.5)
    else:      sleep (0.2)
    print ("   s = %3.1f" % (s,))
    # Measure.
    J.process()
    J.wait()
    # Plot result.
    Spec = np.fft.rfft (Ain1)
    ax.plot (Freq, 20 * np.log10 (np.abs (Spec) + 1e-10), color = 'r', lw = 1)
ax.plot ([20, 20e3], [0, 0], color= 'k', lw = 1)
ax.text (125, 22, "freq = 5 kHz, gain = +20 dB, shape = 0...1")

del F, J

# Show result.
#
plt.show()



#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2020 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


# PyQt5 K-meters


from math import log10
from PyQt5 import QtGui, QtCore, QtWidgets


def makek20pixmap0 (sx, sy, m):
    M = QtGui.QPixmap (sx, sy + 2 * m);
    P = QtGui.QPainter (M)
    G = QtGui.QLinearGradient (0, sy + m, 0, m)
    G.setColorAt (0.00, QtGui.QColor (50, 50, 120))
    G.setColorAt (0.20, QtGui.QColor (0, 70, 90))
    G.setColorAt (0.55, QtGui.QColor (0, 90, 0))
    G.setColorAt (0.63, QtGui.QColor (100, 100, 0))
    G.setColorAt (1.00, QtGui.QColor (100, 30, 0))
    P.setPen(QtCore.Qt.NoPen)
    P.setBrush(G)
    P.drawRect (0, 0, sx, sy + 2 * m)
    return M        


def makek20pixmap1 (sx, sy, m):
    M = QtGui.QPixmap (sx, sy + 2 * m);
    P = QtGui.QPainter (M) 
    G = QtGui.QLinearGradient (0, sy + m, 0, m)
    G.setColorAt (0.00, QtGui.QColor (90, 90, 255))
    G.setColorAt (0.20, QtGui.QColor (0, 190, 190))
    G.setColorAt (0.55, QtGui.QColor (0, 255, 0))
    G.setColorAt (0.63, QtGui.QColor (255, 255, 0))
    G.setColorAt (0.75, QtGui.QColor (255, 128, 50))
    G.setColorAt (1.00, QtGui.QColor (255, 50, 50))
    P.setPen(QtCore.Qt.NoPen)
    P.setBrush(G)
    P.drawRect (0, 0, sx, sy + 2 * m)
    return M        


def makek20pixmap2 (sx, sy, m, bg):
    M = QtGui.QPixmap (sx, sy + 2 * m);
    P = QtGui.QPainter (M)
    P.setPen(QtCore.Qt.NoPen)
    P.setBrush(bg)
    P.drawRect (0, 0, sx, sy + 2 * m)
    S = [((120, 120, 255), (0.0, 1.00e-3, 3.16e-3)),
         ((  0, 255,   0), (1.00e-2, 3.16e-2)),
         ((255, 255,   0), (1.00e-1,)),
         ((255,  50,  50), (3.16e-1, 1.00))]
    P.setBrush (QtCore.Qt.NoBrush)
    for X in S:
        C = X [0]
        L = X [1];
        P.setPen (QtGui.QColor (C [0], C [1], C [2]))   
        for v in L:
           d = sy + m - K20mapfunc (v)
           P.drawLine (0, d, sx, d)
    return M        


def makek20pixmap3 (sx, sy, m, bg):
    M = QtGui.QPixmap (sx, sy + 2 * m);
    P = QtGui.QPainter (M)
    P.setPen(QtCore.Qt.NoPen)
    P.setBrush(bg)
    P.drawRect (0, 0, sx, sy + 2 * m)
    F = QtGui.QFont ('Sans', 7, QtGui.QFont.Normal)
    P.setFont (F)
    dx = sx / 2
    dy = sy + m
    P.setPen (QtGui.QColor (140, 140, 255))
    drawctext (P, dx, dy - K20mapfunc (1.00e-3), "-40");
    drawctext (P, dx, dy - K20mapfunc (3.16e-3), "-30");
    P.setPen (QtGui.QColor (0, 255, 0))
    drawctext (P, dx, dy - K20mapfunc (1.00e-2), "-20");
    drawctext (P, dx, dy - K20mapfunc (3.16e-2), "-10");
    P.setPen (QtGui.QColor (255, 255, 0))
    drawctext (P, dx, dy - K20mapfunc (1.00e-1), "0");
    P.setPen (QtGui.QColor (255, 80, 80))
    drawctext (P, dx, dy - K20mapfunc (3.16e-1), "10");
    drawctext (P, dx, dy - K20mapfunc (1.00), "20");
    return M


def drawctext (P, x, y, s):
    M = P.fontMetrics ()
    w = M.width (s)
    P.drawText (QtCore.QPointF (x - w / 2, y + M.ascent () // 2), s)


def K20mapfunc (v):
    if v < 1e-3: return int (12e3 * v)
    v = log10 (v) + 3
    if v < 2.0: return  int (12.1 + v * (50 + v * 8))
    if v > 3.05: v = 3.05;
    return int (v * 80 - 16)


def makek20metrics (bg):
    return ((makek20pixmap0 (8, 224, 6),
             makek20pixmap1 (8, 224, 6),
             makek20pixmap2 (8, 224, 6, bg),
             makek20pixmap3 (18, 224, 6, bg)),
             K20mapfunc, 6)

    
class K20meter (QtWidgets.QWidget):

    def __init__(self, parent, metrics, nn, d1, d2, label = None):
        super (K20meter, self).__init__(parent)
        self.pixmaps = metrics [0]
        self.mapfunc = metrics [1]
        self.margin = metrics [2]
        self.label = label
        self.nn = nn
        self.d1 = d1
        self.d2 = d2
        self.d3 = self.pixmaps [3].width () 
        self.dy = self.pixmaps [0].height ()
        self.sx = sum (nn) * (d1 + d2) + len (nn) * (d2 + self.d3) + self.d3 + 20
        self.sy = self.dy + 40
        self.rms = [0 for i in range (sum (nn))]
        self.pks = [0 for i in range (sum (nn))]
        self.resize (self.sx, self.sy)
        

    def paintEvent (self, ev):
        qp = QtGui.QPainter()
        qp.begin(self)
        qp.setPen (QtCore.Qt.NoPen)
        qp.setBrush (QtGui.QColor (255, 255, 255))
        d1 = self.d1
        d2 = self.d2
        d3 = self.d3
        i = 0
        x = 10
        y = 20
        qp.drawPixmap (x, y, self.pixmaps [3], 0, 0, d3, self.dy)
        x += d3
        for n in self.nn:
            for j in range (n):
                qp.drawPixmap (x, y, self.pixmaps [2], 0, 0, d2, self.dy)
                x += d2
                self.drawmeter (x, y, qp, i + j)
                x += d1
            qp.drawPixmap (x, y, self.pixmaps [2], 0, 0, d2, self.dy)
            x += d2
            i += n
            qp.drawPixmap (x, y, self.pixmaps [3], 0, 0, d3, self.dy)
            x += d3
        qp.end()


    def drawmeter (self, x, y, qp, ind):
        dx = self.d1
        dy1 = self.mapfunc (self.rms [ind]) + self.margin
        dy2 = self.mapfunc (self.pks [ind]) + self.margin + 1
        qp.drawPixmap (x, y + self.dy - dy1, self.pixmaps [1], 0, self.dy - dy1, dx, dy1)
        qp.drawPixmap (x, y, self.pixmaps [0], 0, 0, dx, self.dy - dy1)
        if dy2 > max (40, dy1): qp.drawRect (x, y + self.dy - dy2, dx, 3)
                      




#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import numpy as np
import matplotlib.pyplot as plt
from math import *
from time import sleep
from zita_jacktools.jacksignal import JackSignal


# ----------- Oldfashioned frequency response using sweep and measure  -------------
#
# Make sure jnoisemeter is running, using
# input 1, and select the A, C or ITU filter.


# Generate a logarithmic sweep signal.
#
def logsweep (fmin, fmax, amp, dur, fsamp):

    # Create arrays for input, output and frequency.
    # Padding is added for the filtering in detect().
    len = int (dur * fsamp + 0.1)
    pad = int (0.1 * fsamp + 0.1)
    A = np.zeros ([len + 2 * pad], dtype = np.float32)
    B = np.zeros ([len + 2 * pad], dtype = np.float32)
    F = np.logspace (log10 (fmin), log10 (fmax), len)
    p = 0
    for i in range (len):
        A [i + pad] = amp * sin (2 * pi * p)
        p += F [i] / fsamp
        if p > 1.0: p -= 1.0
    return (A, B, F, pad)    


# Detect RMS and convert to dB.
#
def detect (A, fsamp, tfilt):

    # Two times square for RMS.
    R = 2 * A * A
    # Filter the result. We use a first order
    # lowpass filter, first in forward direction
    # and then in reverse. The result is a linear
    # phase 2nd order filter (i.e. with no delay).
    n = A.shape [0]
    w = 1.0 / (tfilt * fsamp)
    p = 0
    for i in range (n):
        p += w * (R [i] - p)
        R [i] = p
    p = 0    
    for i in range (n):
        j = n - 1 - i
        p += w * (R [j] - p)
        R [j] = 10 * log10 (p + 1e-60)
    return R    


# Create a JackSignal object and connect.
#
J = JackSignal("JackSignal")
if J.get_state() < 0:
    print ("Failed to create JackSignal -- is Jack running ?")
    exit(1)
J.create_output (0, "out-1")
J.create_input (0, "in-1")
J.silence()
J.connect_output (0, "jnoisemeter:in_1")
J.connect_input (0, "jnoisemeter:out")

# Get Jack info.
#
name, fsamp, period = J.get_jack_info ()

# Set parameters.
#
# The amount of detail in the FR depends on the ratio of
# frequency range to sweep time, and on the time constant
# of the detector filter. Smaller values for the latter
# will produce ripples in the LF range.
#
fmin = 15.0
fmax = 22e3
level = 1.0
tsweep = 10.0
tfilt = 0.02

# Generate data.
#
print ("Preparing...")
Aout, Ainp, Freq, pad = logsweep (fmin, fmax, level, tsweep, fsamp)
J.set_output_data (0, Aout)
J.set_input_data (0, Ainp, nskip = period) # Skip one period.

# Run the test
#
print ("Measuring...")
J.process()
J.wait()
del J

# Process the result.
#
print ("Processing...")
Resp = detect (Ainp, fsamp, tfilt)

# Display the FR.
#
print ("Displaying...")
fig = plt.figure (figsize=(8,6), facecolor='white')
ax = fig.add_axes ([0.07, 0.05, 0.9, 0.9])
ax.set_xlim (2e1, 2e4)
ax.set_ylim (-60, 20)
ax.set_xscale ('log')
# Remove padding, and decimate for faster display.
n = int (Freq.shape [0] / 5000)
ax.plot (Freq [0:-1:n], Resp [pad:-pad:n], color='b', lw=1)
ax.grid ()
plt.show()




# ----------------------------------------------------------------------------
#
#  Copyright (C) 2010-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from zita_jacktools.jackclient import JackClient
from zita_jacktools import jackpeaklim_ext


class JackPeaklim(JackClient):

    """
    Multichannel look-ahead peak limiter, equivalent to zita-dpl1. 

    Channels are coupled and will have the same gain at all times.
    This is the type of limiter used to avoid downstream clipping.
    It can be safely used with a short release time. If the limiting
    action is due to low frequency content, a longer release is used
    automatically to avoid following the waveform rather than its
    envelope.
    """

    def __init__(self, nchan, client_name, server_name = None):
        """
        Create a new JackPeaklim instance with 'nchan' channels.

        The result should be checked using get_state().
        The optional 'server_name' allows to select between running
        Jack servers.
        """
        assert (nchan > 0)
        self._jpeaklim, base = jackpeaklim_ext.makecaps (self, client_name, server_name, nchan)
        super().__init__(base)


    def set_inpgain(self, inpgain) :
        """
        Set input gain in dB.

        The range is -30 to +30 dB, the intial value is 0 dB
        """
        return jackpeaklim_ext.set_inpgain (self._jpeaklim, inpgain)


    def set_threshold(self, threshold) :
        """
        Set threshold in dB.

        To accomodate inter-sample peaks on typical program
        material, set this 0.3 dB below the required maximum
        output sample level. The initial value is 0 dB.
        """
        return jackpeaklim_ext.set_threshold (self._jpeaklim, threshold)


    def set_release(self, release) :
        """
        Set release time in seconds.

        The range is 1 millisecond to 1 second. The initial
        release time is 10 ms.
        """
        return jackpeaklim_ext.set_release (self._jpeaklim, release)





#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2013-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import sys
import matplotlib.pyplot as plt
import numpy as np
from time import sleep
from math import hypot, log10
from zita_jacktools.jacksignal import JackSignal
sys.path.append ('..')
from utils.sinewave import *


# ------------------ Harmonic distortion measurement ------------------------
#
# Run jack_nonlin with arguments 0.01 0.01 0

# Test levels (dB) 
#
Levels = [ -50, -45, -40, -35, -30, -25, -20, -15, -10, -5, 0 ]

# Test frequency (Hz) 
#
Freq = 1003

# Time for each test (s)
#
Mtime = 1.0

# Create a Jacksignal object.
#
J = JackSignal("IMtest")
if J.get_state() < 0:
    print ("Failed to create JackSignal -- is the server running ?")
    exit(1)

# Get Jack info.
#
name, Fsamp, period = J.get_jack_info()

# Create ports and connect
#
J.create_output (0, "out")
J.create_input (0, "in")
J.silence()
J.connect_output (0, "jack_nonlin:in")
J.connect_input (0, "jack_nonlin:out")

# Length of test signals, half a second.
#
siglen = int (Mtime * Fsamp + 0.5)
margin = 1000
buflen = siglen + margin

# Generate test signal, will be scaled for output.
# Cast from double to float for Jacksignal.
#
Aout = gen_sinewave (1.00, Freq, Fsamp, buflen).astype (np.float32)

# Create input buffer.
#
Ainp = np.zeros ((buflen,), dtype = np.float32)

# Generate reference signals for selective measurement.
#
M1 = gen_complex (Freq, Fsamp, siglen)
M2 = gen_complex (2 * Freq, Fsamp, siglen)
M3 = gen_complex (3 * Freq, Fsamp, siglen)


# Run test.
#
N = len (Levels)
HD2 = [ 0 for i in range (N) ]
HD3 = [ 0 for i in range (N) ]

for i in range (N):

    # Define signal buffers and run test. 
    amp = pow (10.0, Levels [i] / 20.0) 
    J.set_output_data (0, amp * Aout)
    J.set_input_data (0, Ainp) 
    J.process()
    J.wait()

    # Skip margin samples and measure.
    # We are not interested in the phase here.
    #
    T = Ainp [margin:]
    L1,p = sigdetect (T, M1)
    L2,p = sigdetect (T, M2)
    L3,p = sigdetect (T, M3)

    # Print results.
    #
    L1dB = 20 * log10 (L1)
    HD2 [i] = hd2 = 100 * L2 / L1
    HD3 [i] = hd3 = 100 * L3 / L1
    print ("Level = %5.1f    A1 = %5.1f   hd2 = %6.3f%%  hd3 = %6.3f%%" % (Levels [i], L1dB, hd2, hd3))

# Stop the Jacksignal process.    
del J    

# Create a nice graph to present the results.
#
Levels = np.array (Levels) # Allows to add offsets.
fig = plt.figure (figsize=(8,5), facecolor='white')
ax = fig.add_axes ([0.07, 0.06, 0.86, 0.88])
ax.set_title ("%% Harmonic distortion, %3.1f Hz"  % (Freq,))
ax.set_xlim (-55, 5)
ax.set_ylim (1e-3, 1e1)
ax.set_yscale ('log')
ax.bar (Levels - 0.9, HD2, 0.7, color = 'orange')
ax.bar (Levels + 0.1, HD3, 0.7, color = 'red')
ax.text (-51, 5.0, 'HD2', size = 17, color = 'orange')
ax.text (-46, 5.0, 'HD3', size = 17, color = 'red')
ax.grid ()
plt.show ()
    



# ----------------------------------------------------------------------------
#
#  Copyright (C) 2018-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import globdef as G
from PyQt5 import QtGui, QtCore, QtWidgets
from gmsbase import *
from buttonbase import *
from controls import *


class Outputelm (Gmsbase):

    def __init__(self, parent, bstyle, rstyle, callb, index, label): 
        super (Outputelm, self).__init__(parent, callb, index)
        self.resize (G.OP_XS, G.ELM_YS)
        R = Rgainctl (self, rstyle)
        R.move (29, 2)
        R.valueEvent.connect (self.value_event)
        R.show ()
        self.gainctl = R
        L = QtWidgets.QLabel (self)
        L.setFont (G.VAL_FONT)
        L.setText ('Off')
        L.setGeometry (20, 30, G.ELM_XS - 8, 14)
        L.setAlignment (QtCore.Qt.AlignRight)
        self.gaintxt = L
        L = QtWidgets.QLabel (self)
        L.setFont (G.LAB_FONT)
        L.setText (label)
        L.setGeometry (61, 10, G.OP_XS - 65, 20)
        L.setAlignment (QtCore.Qt.AlignLeft)
        y = G.ELM_YS // 2 + 1
        self.bmute = ButtonBase (self, bstyle, True, 0)
        self.bmute.set_text ("M")
        self.bmute.move (6, y - 17)
        self.bmute.bpressEvent.connect (self.mute_event)
        self.bsolo = ButtonBase (self, bstyle, True, 1)
        self.bsolo.set_text ("S")
        self.bsolo.move (6, y + 1)
        self.bsolo.bpressEvent.connect (self.solo_event)

     
        
class Outputwin (QtWidgets.QWidget):

    def __init__(self, parent, nelm, callb, labels = None): 
        super (Outputwin, self).__init__(parent)
        self.nelm = nelm
        self.xs = G.OP_XS
        self.ys = nelm * (G.ELM_YS + 1) + 1
        self.resize (self.xs, self.ys)
        pal = self.palette () 
        bgc = pal.color (pal.Window)
        bstyle = ButtonStyle (17, 17, bgc, [G.FG_N,G.COL5,G.COL2], [G.MK_N,G.COL0,G.COL9], G.BUT_FONT, 9)
        rstyle = RotaryStyle (11, bgc, [G.FG_N,G.COL4], [G.MK_N,G.COL0])
        self.controls = [ None for i in range (nelm) ]
        y = 1
        for i in range (nelm):
            label = '%d' % (i,) if labels is None else labels [i]
            E = Outputelm (self, bstyle, rstyle, callb, i, label)  
            E.move (0, y)
            self.controls [i] = E
            y += G.ELM_YS + 1

    def get_state (self, elm):
        return self.controls [elm].get_state ()

    def set_state (self, elm, gaindb, mute):
        return self.controls [elm].set_state (gaindb, mute)

    def get_solo (self, elm):
        return self.controls [elm].get_solo ()

    def any_solo (self):
        for E in self.controls:
            if E.get_solo (): return True
        return False    

    def exc_solo (self, elm):
        for i in range (self.nelm):
            if i != elm: self.controls [i].solo_off ()

    def paintEvent(self, E):
        qp = QtGui.QPainter(self)
        qp.setRenderHint (QtGui.QPainter.Antialiasing)
        qp.setBrush (QtCore.Qt.NoBrush)
        qp.setPen (QtGui.QPen (G.MK_N, 0.2))
        qp.translate (0.5, 0.5)
        y = 0
        for i in range (self.nelm + 1):
            qp.drawLine (0, y, self.xs, y)
            y += G.ELM_YS + 1

